import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;

public class MountainCanvas extends BCanvas{
  public static boolean debug = true;
  public static final int WIRE_FRAME = 0;
  public static final int ZBUF       = 1;
  public static final int PERS       = 0;
  public static final int PARA       = 1;

  public Color back_color = Color.black;
  public Color wire_color = Color.white;
  public Color mount_color = new Color(160,170,170);

  public int Ia = (new Color(80,85,85)).getRGB();
  public int Il = (new Color(255,255,255)).getRGB();
  public Vec  L = new Vec(0.0,-1.0, 0.0);


  public MountainStrip   mount;
  public double rot_x = Math.PI/12.0;
  public double rot_y = Math.PI/6.0; 
  public double init_vp  = 2000;
  public double init_prp = 2400;
  public double vp  = init_vp;
  public double prp = init_prp;
  public int    prev_draw_mode = WIRE_FRAME;
  public int    draw_mode      = WIRE_FRAME;
  public int    proj_mode      = PERS;
  public double surface_roughness = 0.13;
  public double drot = Math.PI/20.0;
  
  public int mouse_d_x;
  public int mouse_d_y;

  public int orgx, orgy;
  public int width;
  public int height;

  public MountainCanvas(int width, int height){
    super(width, height);
    this.width = width;
    this.height = height;
    mount     = new MountainStrip(-1000,-1000,1000,1000, -50, 500, 3,3);
    mount.setAllColor(mount_color);
    mount.randomizeElevation(surface_roughness);
    mount.yAxisRotate(rot_y);
    mount.xAxisRotate(rot_x);
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
  }

  public void paint(Graphics g){
    if( offs != null ){
      g.drawImage(offs,0,0,this);
    }
  }
    
  public void makeOffScreen(){
    offs  = this.createImage(size().width, size().height);
    offsg = offs.getGraphics();
    /* $BCf1{$,(B(0,0) $B$K$J$k$h$&$K$7$F$*$/(B */
    orgx = size().width/2;
    orgy = size().height/2;

    offsg.translate(orgx, orgy);
  }
 
  public void draw_clear(){
    offsg.setColor(back_color);
    // jdk1.1 $B$G$O(B size() --> getSize()
    offsg.fillRect(-orgx,-orgy, size().width, size().height);
  }


  public void draw(){
    if(draw_mode == WIRE_FRAME){
      offsg.setColor(wire_color);
      mount.drawWireFrame(offsg);
    }
    else if( draw_mode == ZBUF){
      MemoryImageSource mis =
	mount.drawZBuffer(offsg, -10000000.0, back_color.getRGB(),
			  - width/2, - height/2,
			  width/2,  height/2,
			  Ia, Il, L,vp,prp);
      Toolkit tk = Toolkit.getDefaultToolkit();
      Image image = tk.createImage(mis);
      offsg.drawImage(image,- width/2, - height/2,this);
    }
  }
			
 

  public void clear_mountain(){
    mount     = new MountainStrip(-1000,-1000,1000,1000, -50, 500, 3,3);
   
    mount.randomizeElevation(surface_roughness);
    mount.setAllColor(mount_color);
    mount.yAxisRotate(rot_y);
    mount.xAxisRotate(rot_x);
   
    vp  = init_vp;
    prp = init_prp;
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public void set_roughness(double roughness){
    surface_roughness = roughness;
  }

  public void divide(){
    mount.divide(surface_roughness);
    mount.setAllColor(mount_color);
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
       
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public void zoom(double z){
    vp  -= z;
    prp -= z;
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
    
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public void setDrawMode(int mode){
    draw_mode = mode;
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public void setProjMode(int mode){
    proj_mode = mode;
     if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public boolean mouseDown(Event e, int x, int y){
    prev_draw_mode = draw_mode;
    mouse_d_x = x;
    mouse_d_y = y;
    return false;
  }

  public boolean mouseDrag(Event e, int x, int y){
    draw_mode = WIRE_FRAME;

    double m = Math.abs( (y - mouse_d_y)/(double)(x - mouse_d_x) );
    if( m > 1 ){
      if( y - mouse_d_y < 0 ){
	mount.xAxisRotate( - drot );
      }
      else{
	mount.xAxisRotate(  drot );
      }
    }
    else{
      if( x - mouse_d_x < 0 ){
	mount.yAxisRotate( - drot );
      }
      else{
	mount.yAxisRotate( drot );
      }
    }
   
  
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
   
    draw_clear();
    draw();
    paint(getGraphics());
    mouse_d_x = x;
    mouse_d_y = y;
    
    return false;
  }

  public boolean mouseUp(Event e, int x, int y){
    draw_mode = prev_draw_mode;
    draw_clear();
    draw();
    paint(getGraphics());
    return false;
  }
  
}

