import java.awt.*;
import java.lang.*;
import java.util.*;

public class MountainStrip extends TriangleStrip{
  public Vertex[][] elevation;
  public int n, m;

  public double     min_height;
  public double     max_height;
  
  /* (minx,0,minz)-(maxx,0,minz)-(maxx,0,maxz)-(minx,0,maxz)
     $B$r%Y!<%9$K$7$F(Bn x m$B$N%a%C%7%e$r:n$k(B
     y$B:BI8$,;3$N9b$5(B */
  
  public MountainStrip(double minx, double minz,
		       double maxx, double maxz,
		       double miny, double maxy, 
		       int    n, int m){
    this.n = n;
    this.m = m;
    min_height = miny;
    max_height = maxy;
    
    elevation = new Vertex[n][m];
    double dx = (maxx - minx)/(double)(n-1);
    double dz = (maxz - minz)/(double)(m-1);
    
    for(int i = 0; i < n; i++){
      for(int j = 0; j < m; j++){
	elevation[j][i] = new Vertex(minx+i*dx,0,minz+j*dz);
      }
    }
    
    initMesh(elevation);
  }
  
  public void initMesh(Vertex[][] vertex_array){
    vertex_list  = new Vector();
    edge_list    = new Vector();
    surface_list = new Vector();
    Vertex v1,v2,v3,v4;
    Edge   e1,e2,e3,e4,e5;
    Triangle s1,s2;
    int ar = vertex_array.length; // >= 2
    int ac = vertex_array[0].length; // >= 2
    int al = ar*ac;
    Edge[][] h_edges = new Edge[ar][ac-1];
    Edge[][] v_edges = new Edge[ar-1][ac];
    Edge[][] o_edges = new Edge[ar-1][ac-1];

    for(int i = 0; i < ar; i++){
      for(int j = 0; j < ac; j++){
	vertex_list.addElement(vertex_array[i][j]);
      }
    }
    
    for(int i = 0; i < ar; i++){
      for(int j = 0; j < ac-1; j++){
	h_edges[i][j] = new Edge(vertex_array[i][j], vertex_array[i][j+1]);
	edge_list.addElement(h_edges[i][j]);
      }
    }
    
    for(int i = 0; i < ar-1; i++){
      for(int j = 0; j < ac; j++){
	v_edges[i][j] = new Edge(vertex_array[i][j], vertex_array[i+1][j]);
	edge_list.addElement(v_edges[i][j]);
      }
    }

    for(int i = 0; i < ar-1; i++){
      for(int j = 0; j < ac-1; j++){
	if( i % 2 == 0){
	  o_edges[i][j] = new Edge(vertex_array[i][j], vertex_array[i+1][j+1]);
	}
	else{
	  o_edges[i][j] = new Edge(vertex_array[i][j+1], vertex_array[i+1][j]);
	}
	edge_list.addElement(o_edges[i][j]);
      }
    }
    
    for(int i = 0; i < ar - 1; i++){
      for(int j = 0; j < ac - 1; j++){

	v1 = vertex_array[i][j];
	v2 = vertex_array[i][j+1];
	v3 = vertex_array[i+1][j+1];
	v4 = vertex_array[i+1][j];

	e1 = h_edges[i][j];
	e2 = v_edges[i][j+1];
	e3 = h_edges[i+1][j];
	e4 = v_edges[i][j];
	e5 = o_edges[i][j];

	if( i % 2 == 0){
	  s1 = new Triangle(e1,e2,e5,v1,v3,v2);
	  s2 = new Triangle(e5,e3,e4,v1,v4,v3);
	}
	else{
	  s1 = new Triangle(e1,e5,e4,v1,v4,v2);
	  s2 = new Triangle(e2,e3,e5,v2,v4,v3);
	}
	surface_list.addElement(s1);
	surface_list.addElement(s2);
      }
    }
  }

  /* $B3FD:E@$N(B($B86J?LL$+$i$N(B)$B9b$5$r%i%s%@%`$K(B [min_height,max_heigth]
     $B$NHO0O$GJQ2=$5$;$k!#(B*/
 
  public void randomizeElevation(double roughness){
    Random rand = new Random();
    Vertex v;
    Rotation rot = getRot();
   
    double r;

    for(int i = 0; i < vertex_list.size(); i++){
      v = (Vertex)vertex_list.elementAt(i);
      v.returnToInit();
      r = min_height + rand.nextGaussian() * (max_height - min_height) * roughness; 
      v.y = r;
      v.setCurrentAsInit();
      v.xAxisRotate(rot.x);
      v.yAxisRotate(rot.y);
      v.zAxisRotate(rot.z);
    }
  }
    
  public void divide(double sr){
    /* sr : surface roughness */
    Rotation rot = getRot();
    returnToInit();

      
    Random rand = new Random();
    Vertex a,b,c,d,e,f,g,h,mid;
    double r;
    double dx,dz;
    int     ti,tj,tip,tipp,tjp,tjpp;

    Vertex[][] new_elevation = new Vertex[2*n-1][2*m-1];
   
    for(int i = 0; i < n; i++){
      for(int j = 0; j < m; j++){
	new_elevation[j*2][i*2] = elevation[j][i];
      }
    }

    for(int i = 1; i < 2*n - 1; i +=2){
      for(int j = 1; j < 2*m - 1; j +=2){
	a = new_elevation[j-1][i-1];
	b = new_elevation[j-1][i+1];
	c = new_elevation[j+1][i+1];
	d = new_elevation[j+1][i-1];
	
	dx = b.x - a.x;
	dz = d.z - a.z;
		
	if(new_elevation[j-1][i] == null){
	  r = rand.nextGaussian() * sr * dx;
	  e = new Vertex( (a.x+b.x)/2.0, (a.y+b.y)/2.0 + r, a.z );
	  new_elevation[j-1][i] = e;
	}
	
	if(new_elevation[j][i+1] == null){
	  r = rand.nextGaussian() * sr * dz;
	  f = new Vertex( b.x, (b.y+c.y)/2.0 + r, (b.z+c.z)/2.0 );
	  new_elevation[j][i+1] = f;
	}

	if(new_elevation[j+1][i] == null){
	  r = rand.nextGaussian() * sr * dx;
	  g = new Vertex( (a.x+b.x)/2.0, (c.y+d.y)/2.0 + r, c.z );
	  new_elevation[j+1][i] = g;
	}

	if(new_elevation[j][i-1] == null){
	  r = rand.nextGaussian() * sr * dz;
	  h = new Vertex( d.x, (d.y+a.y)/2.0 + r, (d.z+a.z)/2.0 );
	  new_elevation[j][i-1] = h;
	}

	r = rand.nextGaussian() * sr * (dx+dz)/2.0;
	mid = new Vertex((a.x+b.x)/2.0, 
		       (a.y+b.y+c.y+d.y)/4.0 + r, 
		       (d.z+a.z)/2.0);
	new_elevation[j][i] = mid;

      }
    }
    
    elevation = new_elevation;
    n = 2*n-1;
    m = 2*m-1;
    initMesh(elevation);
    xAxisRotate(rot.x);
    yAxisRotate(rot.y);
   
  }
}
